#include <DXUT.h>
#include "__CSampleUIState.h"
#include "__CSampleUI.h"
#define NUM_MODEL_CHOICES 6

CD3DSettingsDlg				g_D3DSettingsDlg;          // Device settings dialog
__CSampleUIState			g_StateChanged;			// Global Sate

LPCWSTR g_strMeshFile[] =
{
		L"Media\\sphere.x",
		L"Media\\cube.x",
		L"Media\\Dwarf\\dwarf.x",
		L"Media\\ball2.x",
		L"Media\\box.x",
		L"Media\\tiger.x"
};
LPCWSTR g_strMeshChoice[] =
{
		L"sphere",
		L"cube",
		L"dwarf",
		L"ball",
		L"box",
		L"tiger"

};

LRESULT __CSampleUI::Create(ID3D10Device* pd3dDevice)
{
	HRESULT hr;
	V_RETURN( m_DialogResourceManager.OnD3D10CreateDevice( pd3dDevice ) );
	V_RETURN( g_D3DSettingsDlg.OnD3D10CreateDevice( pd3dDevice ) );
	V_RETURN( D3DX10CreateFont( pd3dDevice, 15, 0, FW_BOLD, 1, FALSE, DEFAULT_CHARSET, 
		OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, 
		L"Arial", &m_pFont ) );
	V_RETURN( D3DX10CreateSprite( pd3dDevice, 512, &m_pTextSprite ) );
	m_pTxtHelper = new CDXUTTextHelper( NULL, NULL, m_pFont, m_pTextSprite, 15 );
	return S_OK;

}

void __CSampleUI::Init()
{
//	HRESULT hr;
	ZeroMemory(&g_StateChanged, sizeof(g_StateChanged));
	// Initialize dialogs
	g_D3DSettingsDlg.Init( &m_DialogResourceManager );
	m_HUD.Init( &m_DialogResourceManager );
	m_SampleUI.Init( &m_DialogResourceManager );
	m_DefaultUI.Init( &m_DialogResourceManager );

	m_HUD.SetCallback( OnGUIEvent ); int iX = 15; int iY = 10; 
	m_HUD.AddButton( IDC_TOGGLEFULLSCREEN, L"Toggle full screen", iX, iY, 125, 22 );
	m_HUD.AddButton( IDC_TOGGLEREF, L"Toggle REF (F3)",           iX, iY += 24, 125, 22, VK_F3 );
	m_HUD.AddButton( IDC_CHANGEDEVICE, L"Change device (F2)",     iX, iY += 24, 125, 22, VK_F2 );
	m_HUD.AddButton( IDC_FRACTURE, L"Fracture (F4)",				  iX, iY += 24, 125, 22, VK_F4 );
	iX = 15;
	iY = 10;
	m_SampleUI.SetCallback( OnGUIEvent );
	m_DefaultUI.SetCallback( OnGUIEvent );

	// Title font for static
	m_SampleUI.SetFont( 1, L"Arial", 14, FW_NORMAL );
	CDXUTElement* pElement = m_SampleUI.GetDefaultElement( DXUT_CONTROL_STATIC, 0 );
	if( pElement )
	{
		pElement->iFont = 1;
		pElement->dwTextFormat = DT_RIGHT | DT_VCENTER;
	}

	// Technique
	m_SampleUI.AddStatic( -1, L"Technique", iX, iY += 24, 115, 22 );
	m_SampleUI.AddComboBox( IDC_TECHNIQUE, iX + 125, iY, 150, 22 );
	m_SampleUI.GetComboBox( IDC_TECHNIQUE )->SetScrollBarWidth(0);
	for(int i =0;i<NUM_MODEL_CHOICES;i++)
	{
		m_SampleUI.GetComboBox( IDC_TECHNIQUE )->AddItem( g_strMeshChoice[i], (void*)g_strMeshFile[i] );

	}
	// Animation speed
	iY += 10;
	m_SampleUI.AddStatic( -1, L"Animation Speed", iX, iY += 24, 115, 22 );
	m_SampleUI.AddSlider( IDC_ANIMATION_SPEED, iX + 125, iY, 125, 22, 0, 3000, 700 ); 

	// Color transmission
	iY += 10;
	m_SampleUI.AddStatic( IDC_RED_TRANSMIT_LABEL, L"Num Cracks", iX, iY += 24, 115, 22 );
	m_SampleUI.AddSlider( IDC_RED_TRANSMIT_SLIDER, iX + 125, iY, 125, 22, 1, 50, 10 ); 
	m_SampleUI.AddStatic( IDC_GREEN_TRANSMIT_LABEL, L"Crack Size", iX, iY += 24, 115, 22 );
	m_SampleUI.AddSlider( IDC_GREEN_TRANSMIT_SLIDER, iX + 125, iY, 125, 22, 1, 50, 20 ); 
	m_SampleUI.AddStatic( IDC_BLUE_TRANSMIT_LABEL, L"Largest Crack", iX, iY += 24, 115, 22 );
	m_SampleUI.AddSlider( IDC_BLUE_TRANSMIT_SLIDER, iX + 125, iY, 125, 22, 1, 500, 100 ); 

//--------------------------------------------------------------------------------------
// This section of code was added by the XmlGuihelper project
//--------------------------------------------------------------------------------------
	m_HUD.AddStatic(IDC_LABEL_HEADER_LABEL, L"This is my xml test", 0, 120, 150, 10);
	m_DefaultUI.AddStatic(IDC_LABEL_112, L"my second label", 10, 40, 75, 10);
	m_DefaultUI.AddStatic(IDC_LABEL_102, L"my second label", 10, 75, 75, 10);
	m_DefaultUI.AddButton(IDC_BUTTON_B1, L"Button uno", 10, 110, 150, 25);
	m_DefaultUI.AddButton(IDC_BUTTON_B2, L"Button two", 10, 145, 75, 25);
	m_DefaultUI.AddButton(IDC_BUTTON_105, L"Button three", 10, 180, 75, 10);
	m_DefaultUI.AddCheckBox(IDC_CHECKBOX_106, L"CheckBox XML", 10, 215, 150, 20);
	m_DefaultUI.AddSlider(IDC_SLIDERBAR_107, 10, 250, 125, 35, 0, 100);
	m_DefaultUI.AddSlider(IDC_SLIDERBAR_green_slider, 10, 285, 125, 35, 0, 100);
	m_DefaultUI.AddSlider(IDC_SLIDERBAR_red_slider, 10, 320, 125, 35, 0, 100);
//--------------------------------------------------------------------------------------

}

void __CSampleUI::Destroy()
{
	m_DialogResourceManager.OnD3D10DestroyDevice();
	g_D3DSettingsDlg.OnD3D10DestroyDevice();
	SAFE_RELEASE( m_pFont );
	SAFE_RELEASE( m_pTextSprite );
	SAFE_DELETE	( m_pTxtHelper );

}

BOOL __CSampleUI::MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, bool* pbNoFurtherProcessing, void* pUserContext )
{
	// Always allow dialog resource manager calls to handle global messages
	// so GUI state is updated correctly
	*pbNoFurtherProcessing = m_DialogResourceManager.MsgProc( hWnd, uMsg, wParam, lParam );
	if( *pbNoFurtherProcessing )
		return FALSE;

	if( g_D3DSettingsDlg.IsActive() )
	{
		g_D3DSettingsDlg.MsgProc( hWnd, uMsg, wParam, lParam );
		return FALSE;
	}

	// Give the dialogs a chance to handle the message first
	*pbNoFurtherProcessing = m_HUD.MsgProc( hWnd, uMsg, wParam, lParam );
	if( *pbNoFurtherProcessing )
		return TRUE;
	*pbNoFurtherProcessing = m_SampleUI.MsgProc( hWnd, uMsg, wParam, lParam );
	if( *pbNoFurtherProcessing )
		return TRUE;
	*pbNoFurtherProcessing = m_DefaultUI.MsgProc( hWnd, uMsg, wParam, lParam );
	if( *pbNoFurtherProcessing )
		return TRUE;
	
	return FALSE;
}

void __CSampleUI::OnD3D10SwapChainReleasing( )
{
	m_DialogResourceManager.OnD3D10ReleasingSwapChain();
}
//--------------------------------------------------------------------------------------
// Render the help and statistics text. This function uses the ID3DXFont interface for 
// efficient text rendering.
//--------------------------------------------------------------------------------------
void __CSampleUI::RenderText()
{

	// The helper object simply helps keep track of text position, and color
	// and then it calls pFont->DrawText( m_pTextSprite, strMsg, -1, &rc, DT_NOCLIP, m_clr );
	// If NULL is passed in as the sprite object, then it will work however the 
	// pFont->DrawText() will not be batched together.  Batching calls will improves performance.

	// Output statistics
	m_pTxtHelper->Begin();
	m_pTxtHelper->SetInsertionPos( 5, 5 );
	m_pTxtHelper->SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 0.0f, 1.0f ) );
	m_pTxtHelper->DrawTextLine( DXUTGetFrameStats(true) ); // Show FPS
	m_pTxtHelper->DrawTextLine( DXUTGetDeviceStats() );

	// Draw help
	if( m_bShowHelp )
	{
		m_pTxtHelper->SetInsertionPos( 10, m_backBufWidth-15*8 );
		m_pTxtHelper->SetForegroundColor( D3DXCOLOR( 1.0f, 0.75f, 0.0f, 1.0f ) );
		m_pTxtHelper->DrawTextLine( L"Controls (F1 to hide):" );

		m_pTxtHelper->SetInsertionPos( 40, m_backBufWidth-15*7 );
		m_pTxtHelper->DrawTextLine( L"Rotate Model: Left Mouse" );
		m_pTxtHelper->DrawTextLine( L"Rotate Light: Middle Mouse" );
		m_pTxtHelper->DrawTextLine( L"Rotate Camera and Model: Right Mouse" );
		m_pTxtHelper->DrawTextLine( L"Rotate Camera: Ctrl + Right Mouse" );
		m_pTxtHelper->DrawTextLine( L"Wireframe: W" );
		m_pTxtHelper->DrawTextLine( L"Quit: ESC" );
	}
	else
	{
		m_pTxtHelper->SetInsertionPos( 10, m_backBufWidth-15*2 );
		m_pTxtHelper->SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 1.0f, 1.0f ) );
		m_pTxtHelper->DrawTextLine( L"Press F1 for help" );
	}
	m_pTxtHelper->End();
}
void __CSampleUI::Render(float fElapsedTime)
{
	if(g_D3DSettingsDlg.IsActive())
	{
		g_D3DSettingsDlg.OnRender( fElapsedTime );

	}
	else
	{
		m_HUD.OnRender( fElapsedTime  );
		m_SampleUI.OnRender( fElapsedTime  );
		m_DefaultUI.OnRender( fElapsedTime );
		RenderText();
	}

}
void __CSampleUI::OnD3D10SwapChainResized( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc )
{
	m_DialogResourceManager.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc );
	g_D3DSettingsDlg.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc );

	m_backBufHeight = pBackBufferSurfaceDesc->Height;
	m_backBufWidth	= pBackBufferSurfaceDesc->Width;
	// Create a sprite to help batch calls when drawing many lines of text
	m_HUD.SetLocation( m_backBufWidth-170, 0 );
	m_HUD.SetSize( 170, 170 );
	m_SampleUI.SetLocation( m_backBufWidth-300, m_backBufHeight-245 );
	m_SampleUI.SetSize( 300, 300 );
	m_DefaultUI.SetLocation(0, 0);
	m_DefaultUI.SetSize(m_backBufWidth, m_backBufHeight);

}
void CALLBACK __CSampleUI::OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext  )
{
	switch( nControlID )
	{
//--------------------------------------------------------------------------------------
// This section of code was added by the XmlGuihelper project
//--------------------------------------------------------------------------------------
	case IDC_LABEL_HEADER_LABEL:
/*TODO ADD CALL OR CODE HERE*/
	break;
	case IDC_LABEL_112:
/*TODO ADD CALL OR CODE HERE*/
	break;
	case IDC_LABEL_102:
/*TODO ADD CALL OR CODE HERE*/
	break;
	case IDC_BUTTON_B1:
/*TODO ADD CALL OR CODE HERE*/
	break;
	case IDC_BUTTON_B2:
/*TODO ADD CALL OR CODE HERE*/
	break;
	case IDC_BUTTON_105:
/*TODO ADD CALL OR CODE HERE*/
	break;
	case IDC_CHECKBOX_106:
/*TODO ADD CALL OR CODE HERE*/
	break;
	case IDC_SLIDERBAR_107:
/*TODO ADD CALL OR CODE HERE*/
	break;
	case IDC_SLIDERBAR_green_slider:
/*TODO ADD CALL OR CODE HERE*/
	break;
	case IDC_SLIDERBAR_red_slider:
/*TODO ADD CALL OR CODE HERE*/
	break;

//--------------------------------------------------------------------------------------
	case IDC_TOGGLEFULLSCREEN: DXUTToggleFullScreen(); break;
	case IDC_TOGGLEREF:        DXUTToggleREF(); break;
	case IDC_CHANGEDEVICE:     g_D3DSettingsDlg.SetActive( !g_D3DSettingsDlg.IsActive() ); break;
	case IDC_TECHNIQUE:		   g_StateChanged.b_modelReload = TRUE; break;
	case IDC_FRACTURE:		   g_StateChanged.b_fracture	= TRUE; break;
	}

}

