//--------------------------------------------------------------------------------------
// File: BasicTNL.fx
//
//--------------------------------------------------------------------------------------


struct SIMPLE_VS_OUTPUT
{
    float4 Position   : SV_POSITION;   // vertex position 
    float4 Diffuse    : COLOR0;     // vertex diffuse color (note that COLOR0 is clamped from 0..1)
};


struct SIMPLE_VS_INPUT
{
    float3 Position   : POSITION;   // vertex position 
    float4 Diffuse    : COLOR0;     // vertex diffuse color (note that COLOR0 is clamped from 0..1)
};

struct VS_INPUT
{
    float3 Pos          : POSITION;         //position
    //float3 Norm         : NORMAL;           //normal
    float2 Tex          : TEXCOORD0;        //texture coordinate
};

struct PS_INPUT
{
    float4 Pos : SV_POSITION;
    //float3 Norm : TEXCOORD0;
    float2 Tex : TEXCOORD1;
};

//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
cbuffer globals
{
	float4 g_MaterialAmbientColor;      // Material's ambient color
	float4 g_MaterialDiffuseColor;      // Material's diffuse color
	float3 g_LightDir;                  // Light's direction in world space
	float4 g_LightDiffuse;              // Light's diffuse color
	float4x4 g_mWorld;                  // World matrix for object
	float4x4 g_mWorldViewProjection;    // World * View * Projection matrix
};

//--------------------------------------------------------------------------------------
// Textures and Samplers
//--------------------------------------------------------------------------------------
Texture2D g_txDiffuse;
Texture2DArray g_tx2dArray;

SamplerState g_samLinear
{
    Filter = ANISOTROPIC;
    AddressU = Wrap;
    AddressV = Wrap;
};

Texture1D g_txRandom;
SamplerState g_samPoint
{
    Filter = MIN_MAG_MIP_POINT;
    AddressU = Wrap;
    AddressV = Wrap;
};

SamplerState ScrambleTextureSampler
{
    Filter = MIN_MAG_MIP_POINT;
    AddressU = Clamp;
    AddressV = Clamp;
};

SIMPLE_VS_OUTPUT RenderSimpleVS(  SIMPLE_VS_INPUT input )
{
    SIMPLE_VS_OUTPUT Output;
    
    // Transform the position from object space to homogeneous projection space
    Output.Position = mul(float4(input.Position,1), g_mWorldViewProjection);
    
    // Calc diffuse color    
    Output.Diffuse = input.Diffuse;
    Output.Diffuse.a = 1.0f; 
       
    return Output;    
}

float4 RenderSimplePS( SIMPLE_VS_OUTPUT In )  : SV_Target
{ 
    float4 Output;

    // Lookup mesh texture and modulate it with diffuse
    Output.xyz = In.Diffuse;

    return Output;
}


//--------------------------------------------------------------------------------------
// Vertex Shader
//--------------------------------------------------------------------------------------
PS_INPUT VS( VS_INPUT input )
{
    PS_INPUT output = (PS_INPUT)0;
    //output.Pos = mul( float4(input.Pos,1), g_mWorldViewProjection );
    output.Pos = float4(input.Pos, 1);
    //output.Norm = mul( input.Norm, g_mWorld );
    output.Tex = input.Tex;
    
    return output;
}


//--------------------------------------------------------------------------------------
// Pixel Shader
//--------------------------------------------------------------------------------------
float4 PS( PS_INPUT input) : SV_Target
{
    //calculate lighting assuming light color is <1,1,1,1>
    //float fLighting = saturate( dot( input.Norm, g_LightDir ) );
    float4 outputColor = g_txDiffuse.Sample( g_samLinear, input.Tex );
    outputColor.a = 1;
    return outputColor;
}


technique10 RenderColor
{
    pass P0
    {          
        SetVertexShader( CompileShader( vs_4_0, RenderSimpleVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, RenderSimplePS() ) );
    }
}

technique10 Render
{
    pass P0
    {          
        SetVertexShader( CompileShader( vs_4_0,VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
    }
}

// FFT stuff.......

struct SCR_VS_INPUT
{
	float2 Pos          : POSITION;         //position
};

struct SCR_VS_OUTPUT
{
	float2 Pos          : POSITION;         //position
};

//Texture2D SourceImageTexture;
Texture2D ScrambleTexture; 

float ButterflyPassNumber;

SCR_VS_OUTPUT ScrambleVS( SCR_VS_INPUT In )
{
	SCR_VS_OUTPUT Out;
	Out.Pos = In.Pos;
	
	return Out;	
}

float4 ScramblePS( SCR_VS_OUTPUT In ) : SV_Target
{
	float2 coord = In.Pos;

	// get the scrambled indices, that give the location for a and b for computing a+wb or a-wb		
	float2 Indices = ScrambleTexture.Sample( ScrambleTextureSampler, float2(coord.x, ButterflyPassNumber) );
	//float4 a = g_txDiffuse.Sample( g_samLinear, float2(Indices.x, coord.y) );
	//float4 b = g_txDiffuse.Sample( g_samLinear, float2(Indices.y, coord.y) );
	return float4(Indices.x, Indices.y, 0.0, 0.0);
}

technique10 HorizontalScramble
{
	pass P0
	{
        SetVertexShader( CompileShader( vs_4_0, ScrambleVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, ScramblePS() ) );
	}
}

#if 0
technique10 VerticalScramble
{
	pass P0
	{
		float4 main( PS_INPUT In ) : COLOR
		{
			float2 fromPos;
			fromPos = In.imagePos;
			// scramble the x coordinate
			// fromPos.x gets assigned red channel of texture
			fromPos.y = tex1D(scramble, In.scrambleLoc);
			return tex2D(sourceImage, fromPos);
		}
	}
}
#endif