#include "DXUT.h"
#include "DXUTmisc.h"
#include "SDKmisc.h"

#include "CTexture.h"

CTexture::CTexture() :	m_FileName(NULL),
						m_ShaderResourceView(NULL),
						m_RenderTargetView(NULL)
{
}

CTexture::~CTexture()
{
	Destroy();
}


void CTexture::SetTextureAsRenderTargetAndClear()
{
	static const FLOAT clrColor[] = {0.0, 0.0, 0.0, 0.0};

	m_D3DDevice->OMSetRenderTargets(1, &m_RenderTargetView, NULL);
	m_D3DDevice->ClearRenderTargetView(m_RenderTargetView, clrColor);
}

ID3D10ShaderResourceView* CTexture::GetResourceView()
{
	return m_ShaderResourceView;
}

ID3D10RenderTargetView* CTexture::GetRenderTargetView()
{
	return m_RenderTargetView;
}

HRESULT CTexture1D::CreateShaderResourceView()
{
	if (m_ShaderResourceView)
		return S_OK;

	// Create the shader-resource view
	D3D10_SHADER_RESOURCE_VIEW_DESC srDesc;

	D3D10_TEXTURE1D_DESC desc;
	GetDesc(&desc);
	srDesc.Format = desc.Format;
	srDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE1D;
	srDesc.Texture1D.MostDetailedMip = 0;
	srDesc.Texture1D.MipLevels = 1;

	return m_D3DDevice->CreateShaderResourceView(m_Texture1D, &srDesc, &m_ShaderResourceView);
}

HRESULT CTexture2D::CreateShaderResourceView()
{
	if (m_ShaderResourceView)
		return S_OK;

	// Create the shader-resource view
	D3D10_SHADER_RESOURCE_VIEW_DESC srDesc;

	D3D10_TEXTURE2D_DESC desc;
	GetDesc(&desc);
	srDesc.Format = desc.Format;
	srDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
	srDesc.Texture2D.MostDetailedMip = 0;
	srDesc.Texture2D.MipLevels = 1;

	return m_D3DDevice->CreateShaderResourceView(m_Texture2D, &srDesc, &m_ShaderResourceView);
}

HRESULT	CTexture1D::CreateRenderTargetView()
{
	HRESULT hr;
	D3D10_RENDER_TARGET_VIEW_DESC rtDesc;

	D3D10_TEXTURE1D_DESC desc;
	GetDesc(&desc);
	rtDesc.Format = desc.Format;
	rtDesc.ViewDimension = D3D10_RTV_DIMENSION_TEXTURE1D;
	rtDesc.Texture1D.MipSlice = 0;

	V_RETURN(m_D3DDevice->CreateRenderTargetView(m_Texture1D, &rtDesc, &m_RenderTargetView));
	V_RETURN(CreateShaderResourceView());
	return hr;
}


HRESULT	CTexture2D::CreateRenderTargetView()
{
	HRESULT hr;
	D3D10_RENDER_TARGET_VIEW_DESC rtDesc;

	D3D10_TEXTURE2D_DESC desc;
	GetDesc(&desc);
	rtDesc.Format = desc.Format;
	rtDesc.ViewDimension = D3D10_RTV_DIMENSION_TEXTURE2D;
	rtDesc.Texture2D.MipSlice = 0;

	V_RETURN(m_D3DDevice->CreateRenderTargetView(m_Texture2D, &rtDesc, &m_RenderTargetView));
	V_RETURN(CreateShaderResourceView());
	return hr;
}

HRESULT	CTexture1D::CreateRenderTargetTexture(ID3D10Device* pd3dDevice, UINT Width, UINT Height, DXGI_FORMAT Format)
{
	HRESULT hr;
	D3D10_RENDER_TARGET_VIEW_DESC rtDesc;
	m_D3DDevice = pd3dDevice;

	// Create the render target texture
	D3D10_TEXTURE1D_DESC desc;
	ZeroMemory(&desc, sizeof(desc));
	desc.Width = Width;
	desc.MipLevels = 1;
	desc.ArraySize = 1;
	desc.Format = Format;
	desc.Usage = D3D10_USAGE_DEFAULT;
	desc.BindFlags = D3D10_BIND_RENDER_TARGET | D3D10_BIND_SHADER_RESOURCE;
	//desc.CPUAccessFlags = D3D10_CPU_ACCESS_READ | D3D10_CPU_ACCESS_WRITE;
	hr = pd3dDevice->CreateTexture1D(&desc, NULL, &m_Texture1D);
    if( FAILED( hr ) )
        return hr;
	V_RETURN(CreateShaderResourceView());

	rtDesc.Format = desc.Format;
	rtDesc.ViewDimension = D3D10_RTV_DIMENSION_TEXTURE1D;
	rtDesc.Texture1D.MipSlice = 0;

	V_RETURN(pd3dDevice->CreateRenderTargetView(m_Texture1D, &rtDesc, &m_RenderTargetView));
    pd3dDevice->OMSetRenderTargets(1, &m_RenderTargetView, NULL);

	return hr;
}

HRESULT	CTexture2D::CreateRenderTargetTexture(ID3D10Device* pd3dDevice, UINT Width, UINT Height, DXGI_FORMAT Format)
{
	HRESULT hr;
	D3D10_RENDER_TARGET_VIEW_DESC rtDesc;
	m_D3DDevice = pd3dDevice;

	// Create the render target texture
	D3D10_TEXTURE2D_DESC desc;
	ZeroMemory(&desc, sizeof(desc));
	desc.Width = Width;
	desc.Height = Height;
	desc.MipLevels = 1;
	desc.ArraySize = 1;
	desc.Format = Format;
	desc.SampleDesc.Count = 1;
	desc.Usage = D3D10_USAGE_DEFAULT;
	desc.BindFlags = D3D10_BIND_RENDER_TARGET | D3D10_BIND_SHADER_RESOURCE;
	//desc.CPUAccessFlags = D3D10_CPU_ACCESS_READ | D3D10_CPU_ACCESS_WRITE;
	hr = pd3dDevice->CreateTexture2D(&desc, NULL, &m_Texture2D);
    if( FAILED( hr ) )
        return hr;
	V_RETURN(CreateShaderResourceView());

	rtDesc.Format = desc.Format;
	rtDesc.ViewDimension = D3D10_RTV_DIMENSION_TEXTURE2D;
	rtDesc.Texture2D.MipSlice = 0;

	V_RETURN(pd3dDevice->CreateRenderTargetView(m_Texture2D, &rtDesc, &m_RenderTargetView));
    pd3dDevice->OMSetRenderTargets(1, &m_RenderTargetView, NULL);

	return hr;
}

HRESULT	CTexture1D::CreateEmptyTexture(ID3D10Device* pd3dDevice, UINT Width, UINT Height, DXGI_FORMAT Format)
{
	HRESULT hr;
	m_D3DDevice = pd3dDevice;

	// Create the render target texture
	D3D10_TEXTURE1D_DESC desc;
	ZeroMemory(&desc, sizeof(desc));
	desc.Width = Width;
	desc.MipLevels = 1;
	desc.ArraySize = 1;
	desc.Format = Format;
	desc.Usage = D3D10_USAGE_DEFAULT;
	desc.BindFlags = /*D3D10_BIND_RENDER_TARGET | */D3D10_BIND_SHADER_RESOURCE;

	// m_Texture2D is the render target
	V_RETURN(pd3dDevice->CreateTexture1D(&desc, NULL, &m_Texture1D));

	// Create the shader-resource view
	//pd3dDevice->CreateShaderResourceView(m_Texture2D, NULL, &m_ShaderResourceView);
	D3D10_SHADER_RESOURCE_VIEW_DESC srDesc;
	srDesc.Format = desc.Format;
	srDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE1D;
	srDesc.Texture1D.MostDetailedMip = 0;
	srDesc.Texture1D.MipLevels = 1;

	V_RETURN(pd3dDevice->CreateShaderResourceView(m_Texture1D, &srDesc, &m_ShaderResourceView));

	// create a render target view
	//pd3dDevice->CreateRenderTargetView(m_Texture2D, NULL, &m_RenderTargetView);
	D3D10_RENDER_TARGET_VIEW_DESC rtDesc;
	rtDesc.Format = desc.Format;
	rtDesc.ViewDimension = D3D10_RTV_DIMENSION_TEXTURE1D;
	rtDesc.Texture1D.MipSlice = 0;

	hr = pd3dDevice->CreateRenderTargetView(m_Texture1D, &rtDesc, &m_RenderTargetView);

	return hr;
}

HRESULT	CTexture2D::CreateEmptyTexture(ID3D10Device* pd3dDevice, UINT Width, UINT Height, DXGI_FORMAT Format)
{
	HRESULT hr;
	m_D3DDevice = pd3dDevice;

	// Create the render target texture
	D3D10_TEXTURE2D_DESC desc;
	ZeroMemory(&desc, sizeof(desc));
	desc.Width = Width;
	desc.Height = Height;
	desc.MipLevels = 1;
	desc.ArraySize = 1;
	desc.Format = Format;
	desc.SampleDesc.Count = 1;
	desc.Usage = D3D10_USAGE_DEFAULT;
	desc.BindFlags = D3D10_BIND_RENDER_TARGET | D3D10_BIND_SHADER_RESOURCE;

	// m_Texture2D is the render target
	V_RETURN(pd3dDevice->CreateTexture2D(&desc, NULL, &m_Texture2D));

	// Create the shader-resource view
	//pd3dDevice->CreateShaderResourceView(m_Texture2D, NULL, &m_ShaderResourceView);
	D3D10_SHADER_RESOURCE_VIEW_DESC srDesc;
	srDesc.Format = desc.Format;
	srDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
	srDesc.Texture2D.MostDetailedMip = 0;
	srDesc.Texture2D.MipLevels = 1;

	V_RETURN(pd3dDevice->CreateShaderResourceView(m_Texture2D, &srDesc, &m_ShaderResourceView));

	// create a render target view
	//pd3dDevice->CreateRenderTargetView(m_Texture2D, NULL, &m_RenderTargetView);
	D3D10_RENDER_TARGET_VIEW_DESC rtDesc;
	rtDesc.Format = desc.Format;
	rtDesc.ViewDimension = D3D10_RTV_DIMENSION_TEXTURE2D;
	rtDesc.Texture2D.MipSlice = 0;

	hr = pd3dDevice->CreateRenderTargetView(m_Texture2D, &rtDesc, &m_RenderTargetView);

	return hr;
}

HRESULT	CTexture1D::CreateRWTexture(ID3D10Device* pd3dDevice, UINT Width, UINT Height, DXGI_FORMAT Format)
{
	HRESULT hr;
	m_D3DDevice = pd3dDevice;

	// Create the render target texture
	D3D10_TEXTURE1D_DESC desc;
	ZeroMemory( &desc, sizeof(desc) );
	desc.Width = Width;
	desc.MipLevels = 1;
	desc.ArraySize = 1;
	desc.Format = Format;
	desc.Usage = D3D10_USAGE_DYNAMIC;
	desc.BindFlags = D3D10_BIND_SHADER_RESOURCE;
	desc.CPUAccessFlags = /*D3D10_CPU_ACCESS_READ | */D3D10_CPU_ACCESS_WRITE;

	V_RETURN( pd3dDevice->CreateTexture1D(&desc, NULL, &m_Texture1D) );
	V_RETURN( CreateShaderResourceView() );
	return hr;
}

HRESULT	CTexture2D::CreateRWTexture(ID3D10Device* pd3dDevice, UINT Width, UINT Height, DXGI_FORMAT Format)
{
	HRESULT hr;
	m_D3DDevice = pd3dDevice;

	// Create the render target texture
	D3D10_TEXTURE2D_DESC desc;
	ZeroMemory( &desc, sizeof(desc) );
	desc.Width = Width;
	desc.Height = Height;
	desc.MipLevels = 1;
	desc.ArraySize = 1;
	desc.Format = Format;
	desc.SampleDesc.Count = 1;
	desc.Usage = D3D10_USAGE_DYNAMIC;
	desc.BindFlags = D3D10_BIND_SHADER_RESOURCE;
	desc.CPUAccessFlags = /*D3D10_CPU_ACCESS_READ | */D3D10_CPU_ACCESS_WRITE;

	V_RETURN( pd3dDevice->CreateTexture2D(&desc, NULL, &m_Texture2D) );
	V_RETURN( CreateShaderResourceView() );
	return hr;
}

HRESULT	CTexture1D::LoadFromFile(ID3D10Device* pd3dDevice, WCHAR* fileName)
{
	HRESULT hr;

	m_D3DDevice = pd3dDevice;

    if(m_FileName)
	{
		Destroy();
	}
	m_FileName = new WCHAR[MAX_PATH];
	StringCchCopyW(m_FileName, MAX_PATH, fileName);

	ID3D10Resource *pRes = NULL;

    D3DX10_IMAGE_LOAD_INFO loadInfo;
    loadInfo.Width = D3DX10_DEFAULT;
    loadInfo.Height  = D3DX10_DEFAULT;
    loadInfo.Depth  = D3DX10_DEFAULT;
    loadInfo.FirstMipLevel = 0;
    loadInfo.MipLevels = 1;
    loadInfo.Usage = D3D10_USAGE_DEFAULT; //D3D10_USAGE_STAGING;//D3D10_USAGE_DYNAMIC;;    
	loadInfo.BindFlags = D3D10_BIND_SHADER_RESOURCE;
    loadInfo.CpuAccessFlags = 0; //D3D10_CPU_ACCESS_READ;//0;
    loadInfo.MiscFlags = 0;
    loadInfo.Format = DXGI_FORMAT_FROM_FILE;
    loadInfo.Filter = D3DX10_FILTER_NONE;
    loadInfo.MipFilter = D3DX10_FILTER_NONE;
    loadInfo.pSrcInfo = NULL;

	V_RETURN(D3DX10CreateTextureFromFile(pd3dDevice, m_FileName, &loadInfo, NULL, &pRes, NULL));	
    V_RETURN(pRes->QueryInterface(__uuidof(ID3D10Texture1D), (LPVOID*)&m_Texture1D));
    SAFE_RELEASE(pRes);
    
	hr = CreateShaderResourceView();

    return hr;
}

HRESULT	CTexture2D::LoadFromFile(ID3D10Device* pd3dDevice, WCHAR* fileName)
{
	HRESULT hr;

	m_D3DDevice = pd3dDevice;

    if(m_FileName)
	{
		Destroy();
	}
	m_FileName = new WCHAR[MAX_PATH];
	StringCchCopyW(m_FileName, MAX_PATH, fileName);

	ID3D10Resource *pRes = NULL;

    D3DX10_IMAGE_LOAD_INFO loadInfo;
    loadInfo.Width = D3DX10_DEFAULT;
    loadInfo.Height  = D3DX10_DEFAULT;
    loadInfo.Depth  = D3DX10_DEFAULT;
    loadInfo.FirstMipLevel = 0;
    loadInfo.MipLevels = 1;
    loadInfo.Usage = D3D10_USAGE_DEFAULT;
	loadInfo.BindFlags = D3D10_BIND_SHADER_RESOURCE;
    loadInfo.CpuAccessFlags = 0;
    loadInfo.MiscFlags = 0;
    loadInfo.Format = DXGI_FORMAT_FROM_FILE;
    loadInfo.Filter = D3DX10_FILTER_NONE;
    loadInfo.MipFilter = D3DX10_FILTER_NONE;
    loadInfo.pSrcInfo = NULL;

	V_RETURN(D3DX10CreateTextureFromFile(pd3dDevice, m_FileName, &loadInfo, NULL, &pRes, NULL));	
    V_RETURN(pRes->QueryInterface(__uuidof(ID3D10Texture2D), (LPVOID*)&m_Texture2D));
    SAFE_RELEASE(pRes);
    
	hr = CreateShaderResourceView();

    return hr;
}

const void CTexture1D::GetDesc(D3D10_TEXTURE1D_DESC *desc)
{
	m_Texture1D->GetDesc(desc);
}

const void CTexture2D::GetDesc(D3D10_TEXTURE2D_DESC *desc)
{
	m_Texture2D->GetDesc(desc);
}

void CTexture::Destroy()
{
	SAFE_DELETE(m_FileName);
	SAFE_RELEASE(m_ShaderResourceView);
	SAFE_RELEASE(m_RenderTargetView);
}

