#ifndef __CTEXTURE__
#define __CTEXTURE__

enum TextureType { TEXTURE1D, TEXTURE2D };

class CTexture
{
protected:
	WCHAR* m_FileName;
	TextureType m_TextureType;
	ID3D10Device* m_D3DDevice;
	ID3D10ShaderResourceView* m_ShaderResourceView;
	ID3D10RenderTargetView* m_RenderTargetView;

public:
	CTexture();
	~CTexture();

	virtual HRESULT	CreateEmptyTexture(ID3D10Device* pd3dDevice, UINT Width, UINT Height, DXGI_FORMAT Format) = 0;
	virtual HRESULT	CreateRWTexture(ID3D10Device* pd3dDevice, UINT Width, UINT Height, DXGI_FORMAT Format) = 0;
	virtual HRESULT	CreateRenderTargetView() = 0;
	virtual HRESULT	CreateRenderTargetTexture(ID3D10Device* pd3dDevice, UINT Width, UINT Height, DXGI_FORMAT Format) = 0;
	virtual HRESULT CreateShaderResourceView() = 0;

	virtual void SetTextureAsRenderTargetAndClear();

	virtual HRESULT LoadFromFile(ID3D10Device* pd3dDevice, WCHAR* fileName) = 0;
	void	Destroy();

	ID3D10ShaderResourceView* GetResourceView();
	ID3D10RenderTargetView* GetRenderTargetView();

};

class CTexture1D : public CTexture
{
public:
	ID3D10Texture1D*	m_Texture1D;
						
	CTexture1D() : m_Texture1D(NULL)
	{
		m_TextureType = TEXTURE2D;
	}
	~CTexture1D()
	{
		SAFE_RELEASE(m_Texture1D);
	}

	HRESULT	CreateEmptyTexture(ID3D10Device* pd3dDevice, UINT Width, UINT Height, DXGI_FORMAT Format);
	HRESULT	CreateRWTexture(ID3D10Device* pd3dDevice, UINT Width, UINT Height, DXGI_FORMAT Format);
	HRESULT	CreateRenderTargetView();
	HRESULT	CreateRenderTargetTexture(ID3D10Device* pd3dDevice, UINT Width, UINT Height, DXGI_FORMAT Format);
	HRESULT CreateShaderResourceView();
	HRESULT LoadFromFile(ID3D10Device* pd3dDevice, WCHAR* fileName);

	const void GetDesc(D3D10_TEXTURE1D_DESC *desc);

};

class CTexture2D : public CTexture
{
public:
	ID3D10Texture2D*	m_Texture2D;

	CTexture2D() : m_Texture2D(NULL)
	{
		m_TextureType = TEXTURE2D;
	}
	~CTexture2D()
	{
		SAFE_RELEASE(m_Texture2D);
	}

	HRESULT	CreateEmptyTexture(ID3D10Device* pd3dDevice, UINT Width, UINT Height, DXGI_FORMAT Format);
	HRESULT	CreateRWTexture(ID3D10Device* pd3dDevice, UINT Width, UINT Height, DXGI_FORMAT Format);
	HRESULT	CreateRenderTargetView();
	HRESULT	CreateRenderTargetTexture(ID3D10Device* pd3dDevice, UINT Width, UINT Height, DXGI_FORMAT Format);
	HRESULT CreateShaderResourceView();
	HRESULT LoadFromFile(ID3D10Device* pd3dDevice, WCHAR* fileName);

	const void GetDesc(D3D10_TEXTURE2D_DESC *desc);
};

#endif // __CTEXTURE__