#include <DXUT.h>
#include "CUI.h"
#include "CUIState.h"

CUIState			g_StateChanged;			// Global Sate

#define NUM_MODEL_CHOICES 6

CD3DSettingsDlg				g_D3DSettingsDlg;          // Device settings dialog

LPCWSTR g_strMeshFile[] =
{
		L"Media\\sphere.x",
		L"Media\\cube.x",
		L"Media\\Dwarf\\dwarf.x",
		L"Media\\ball2.x",
		L"Media\\box.x",
		L"Media\\tiger.x"
};
LPCWSTR g_strMeshChoice[] =
{
		L"sphere",
		L"cube",
		L"dwarf",
		L"ball",
		L"box",
		L"tiger"

};

LRESULT CUI::Create(ID3D10Device* pd3dDevice)
{
	HRESULT hr;
	V_RETURN( m_DialogResourceManager.OnD3D10CreateDevice( pd3dDevice ) );
	V_RETURN( g_D3DSettingsDlg.OnD3D10CreateDevice( pd3dDevice ) );
	V_RETURN( D3DX10CreateFont( pd3dDevice, 15, 0, FW_BOLD, 1, FALSE, DEFAULT_CHARSET, 
		OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, 
		L"Arial", &m_pFont ) );
	V_RETURN( D3DX10CreateSprite( pd3dDevice, 512, &m_pTextSprite ) );
	m_pTxtHelper = new CDXUTTextHelper( NULL, NULL, m_pFont, m_pTextSprite, 15 );
	return S_OK;

}

void CUI::Init()
{
	ZeroMemory(&g_StateChanged, sizeof(g_StateChanged));

	g_StateChanged.computeFFTEachFrame = true;

	// Initialize dialogs
	g_D3DSettingsDlg.Init( &m_DialogResourceManager );
	m_HUD.Init( &m_DialogResourceManager );
	m_SampleUI.Init( &m_DialogResourceManager );
	m_DefaultUI.Init( &m_DialogResourceManager );

	m_HUD.SetCallback( OnGUIEvent ); int iX = 15; int iY = 10; 

	iX = 15;
	iY = 10;
	m_SampleUI.SetCallback( OnGUIEvent );
	m_DefaultUI.SetCallback( OnGUIEvent );

	// Title font for static
	m_SampleUI.SetFont( 1, L"Arial", 14, FW_NORMAL );
	CDXUTElement* pElement = m_SampleUI.GetDefaultElement( DXUT_CONTROL_STATIC, 0 );
	if( pElement )
	{
		pElement->iFont = 1;
		pElement->dwTextFormat = DT_RIGHT | DT_VCENTER;
	}

	iY += 10;
	iY += 24;

	// Color transmission
	iY += 10;
	m_SampleUI.AddButton( IDC_FFT_X,			L"FFT_X",		iX+225, iY += 24, 50, 22 ); 
	m_SampleUI.AddButton( IDC_FFT_Y,			L"FFT_Y",		iX+225, iY += 24, 50, 22 ); 
	m_SampleUI.AddButton( IDC_FFT_XY,			L"FFT_XY",		iX+225, iY += 24, 50, 22 ); 
	m_SampleUI.AddButton( IDC_IFFT_XY,			L"IFFT_XY",		iX+225, iY += 24, 50, 22 ); 
	//m_SampleUI.AddButton( IDC_FFT_NONE,			L"GreyScale",	iX+225, iY += 24, 54, 22 ); 
	m_SampleUI.AddButton( IDC_CHANGEDEVICE,		L"ChangeDevice",iX+200, iY += 24, 75, 22 ); 
	m_SampleUI.AddCheckBox( IDC_FFT_EACH_FRAME,	L"Compute FFT each frame", iX+125, iY += 24, 150, 22, true, 0, false, &g_StateChanged.m_CBFFTEachFrame); 
}

void CUI::Destroy()
{
	m_DialogResourceManager.OnD3D10DestroyDevice();
	g_D3DSettingsDlg.OnD3D10DestroyDevice();
	SAFE_RELEASE( m_pFont );
	SAFE_RELEASE( m_pTextSprite );
	SAFE_DELETE	( m_pTxtHelper );
}

BOOL CUI::MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, bool* pbNoFurtherProcessing, void* pUserContext )
{
	// Always allow dialog resource manager calls to handle global messages
	// so GUI state is updated correctly
	*pbNoFurtherProcessing = m_DialogResourceManager.MsgProc( hWnd, uMsg, wParam, lParam );
	if( *pbNoFurtherProcessing )
		return FALSE;

	if( g_D3DSettingsDlg.IsActive() )
	{
		g_D3DSettingsDlg.MsgProc( hWnd, uMsg, wParam, lParam );
		return FALSE;
	}

	// Give the dialogs a chance to handle the message first
	*pbNoFurtherProcessing = m_HUD.MsgProc( hWnd, uMsg, wParam, lParam );
	if( *pbNoFurtherProcessing )
		return TRUE;
	*pbNoFurtherProcessing = m_SampleUI.MsgProc( hWnd, uMsg, wParam, lParam );
	if( *pbNoFurtherProcessing )
		return TRUE;
	*pbNoFurtherProcessing = m_DefaultUI.MsgProc( hWnd, uMsg, wParam, lParam );
	if( *pbNoFurtherProcessing )
		return TRUE;
	
	return FALSE;
}

void CUI::OnD3D10SwapChainReleasing( )
{
	m_DialogResourceManager.OnD3D10ReleasingSwapChain();
}
//--------------------------------------------------------------------------------------
// Render the help and statistics text. This function uses the ID3DXFont interface for 
// efficient text rendering.
//--------------------------------------------------------------------------------------
void CUI::RenderText()
{

	// The helper object simply helps keep track of text position, and color
	// and then it calls pFont->DrawText( m_pTextSprite, strMsg, -1, &rc, DT_NOCLIP, m_clr );
	// If NULL is passed in as the sprite object, then it will work however the 
	// pFont->DrawText() will not be batched together.  Batching calls will improves performance.

	// Output statistics
	m_pTxtHelper->Begin();
	m_pTxtHelper->SetInsertionPos( 5, 5 );
	m_pTxtHelper->SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 0.0f, 1.0f ) );
	m_pTxtHelper->DrawTextLine( DXUTGetFrameStats(true) ); // Show FPS
	m_pTxtHelper->DrawTextLine( DXUTGetDeviceStats() );
	// Draw help
	if( m_bShowHelp )
	{
		m_pTxtHelper->SetInsertionPos( 10, m_backBufWidth-15*8 );
		m_pTxtHelper->SetForegroundColor( D3DXCOLOR( 1.0f, 0.75f, 0.0f, 1.0f ) );
		m_pTxtHelper->DrawTextLine( L"Controls (F1 to hide):" );

		m_pTxtHelper->SetInsertionPos( 40, m_backBufWidth-15*7 );
		m_pTxtHelper->DrawTextLine( L"Rotate Model: Left Mouse" );
		m_pTxtHelper->DrawTextLine( L"Rotate Light: Middle Mouse" );
		m_pTxtHelper->DrawTextLine( L"Rotate Camera and Model: Right Mouse" );
		m_pTxtHelper->DrawTextLine( L"Rotate Camera: Ctrl + Right Mouse" );
		m_pTxtHelper->DrawTextLine( L"Wireframe: W" );
		m_pTxtHelper->DrawTextLine( L"Quit: ESC" );
	}
	else
	{
		m_pTxtHelper->SetInsertionPos( 10, m_backBufWidth-15*2 );
		m_pTxtHelper->SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 1.0f, 1.0f ) );
		m_pTxtHelper->DrawTextLine( L"Press F1 for help" );
	}
	m_pTxtHelper->End();
}
void CUI::Render(float fElapsedTime)
{
	if(g_D3DSettingsDlg.IsActive())
	{
		g_D3DSettingsDlg.OnRender( fElapsedTime );

	}
	else
	{
		m_HUD.OnRender( fElapsedTime  );
		m_SampleUI.OnRender( fElapsedTime  );
		m_DefaultUI.OnRender( fElapsedTime );
		RenderText();
	}

}
void CUI::OnD3D10SwapChainResized( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc )
{
	m_DialogResourceManager.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc );
	g_D3DSettingsDlg.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc );

	m_backBufHeight = pBackBufferSurfaceDesc->Height;
	m_backBufWidth	= pBackBufferSurfaceDesc->Width;
	// Create a sprite to help batch calls when drawing many lines of text
	m_HUD.SetLocation( m_backBufWidth-170, 0 );
	m_HUD.SetSize( 170, 170 );
	m_SampleUI.SetLocation( m_backBufWidth-300, m_backBufHeight-245 );
	m_SampleUI.SetSize( 300, 300 );
	m_DefaultUI.SetLocation(0, 0);
	m_DefaultUI.SetSize(m_backBufWidth, m_backBufHeight);

}
void CALLBACK CUI::OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext  )
{
	switch( nControlID )
	{
	case IDC_TOGGLEFULLSCREEN:	DXUTToggleFullScreen(); break;
	case IDC_TOGGLEREF:			DXUTToggleREF(); break;
	case IDC_CHANGEDEVICE:		g_D3DSettingsDlg.SetActive( !g_D3DSettingsDlg.IsActive() ); break;
	case IDC_FFT_X:
		g_StateChanged.fftType = CUIState::FFT_X;
		g_StateChanged.computeFFTEachFrame = true;
		break;
	case IDC_FFT_Y:
		g_StateChanged.fftType = CUIState::FFT_Y;
		g_StateChanged.computeFFTEachFrame = true;
		break;
	case IDC_FFT_XY:
		g_StateChanged.fftType = CUIState::FFT_XY;
		g_StateChanged.computeFFTEachFrame = true;
		break;
	case IDC_IFFT_XY:
		g_StateChanged.fftType = CUIState::IFFT_XY;
		g_StateChanged.computeFFTEachFrame = true;
		break;
	case IDC_FFT_NONE:
		g_StateChanged.fftType = CUIState::FFT_NONE;
		g_StateChanged.computeFFTEachFrame = true;
		break;
	case IDC_FFT_EACH_FRAME:
		g_StateChanged.computeFFTEachFrame = !g_StateChanged.computeFFTEachFrame;		
	}
	g_StateChanged.m_CBFFTEachFrame->SetChecked(g_StateChanged.computeFFTEachFrame);
}
