#ifndef __CUI_H__
#define __CUI_H__

#pragma once
#include "DXUT.h"
#include "DXUTgui.h"
#include "DXUTsettingsdlg.h"
#include "SDKmisc.h"
#include "CUIState.h"

//--------------------------------------------------------------------------------------
// UI control IDs
//--------------------------------------------------------------------------------------
//***IDMARKER
#define IDC_TOGGLEFULLSCREEN          1
#define IDC_TOGGLEREF                 2
#define IDC_CHANGEDEVICE              3
#define IDC_TECHNIQUE                 4
#define IDC_RED_TRANSMIT_SLIDER       5
#define IDC_GREEN_TRANSMIT_SLIDER     6
#define IDC_BLUE_TRANSMIT_SLIDER      7
#define IDC_RED_TRANSMIT_LABEL        8
#define IDC_GREEN_TRANSMIT_LABEL      9
#define IDC_BLUE_TRANSMIT_LABEL       10
#define IDC_LIGHT_SLIDER              11
#define IDC_ENV_SLIDER                12
#define IDC_ENV_LABEL                 13
#define IDC_ANIMATION_SPEED           14
#define IDC_FRACTURE				  15
#define	IDC_FFT_X					  16
#define	IDC_FFT_Y					  17
#define	IDC_FFT_XY					  18
#define	IDC_IFFT_XY					  19
#define IDC_FFT_NONE				  20
#define IDC_FFT_EACH_FRAME			  21

class CUI
{
	CDXUTDialogResourceManager	m_DialogResourceManager; // manager for shared resources of dialogs
	CDXUTDialog					m_HUD;					 // dialog for standard controls
	CDXUTDialog					m_SampleUI;              // dialog for sample specific controls
	CDXUTDialog					m_DefaultUI;              // dialog for sample specific controls
	ID3DX10Font*				m_pFont;				 // Font for drawing text
	ID3DX10Sprite*				m_pTextSprite;			 // Sprite for batching draw text calls
	BOOL						m_bShowHelp;
	int							m_backBufWidth;
	int							m_backBufHeight;
	CDXUTTextHelper*			m_pTxtHelper;

public:
	CUI():m_pFont(NULL), m_pTextSprite(NULL), m_bShowHelp(FALSE) {};
	
	~CUI() { Destroy(); }

	LRESULT Create(ID3D10Device* pd3dDevice);
	void Init();
	void Destroy();
	void Update();
	void Render(float fElapsedTime );
	void RenderText();
	void ToggleHelp(){m_bShowHelp = !m_bShowHelp;};
	void GetSelectedModel( WCHAR* model){ StringCchCopyW(model, MAX_PATH, (WCHAR*)m_SampleUI.GetComboBox(IDC_TECHNIQUE)->GetSelectedData());}
	BOOL MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, bool* pbNoFurtherProcessing, void* pUserContext );
	static void CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext );

	void OnD3D10SwapChainReleasing();
	void OnD3D10SwapChainResized( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc );
};

#endif //__CUI_H__
