#pragma once
#include "CUI.h"
#include "DXUT.h"
#include "DXUTcamera.h"
#include "SDKmisc.h"
#include "CTexture.h"

//enum FFTType { FFT_NONE, FFT_X, FFT_Y, FFT_XY, IFFT_XY };

class FFTDX10
{
	CTexture2D					m_SrcImageTexture;
	CTexture2D					m_PingRTTexture;
	CTexture2D					m_PongRTTexture;
	CTexture2D					m_ButterflyTexture;

	ID3D10ShaderResourceView*	m_FinalRenderedImage;

	CUI					m_UI;
	CModelViewerCamera			m_Camera;               // A model viewing camera

	ID3D10Effect*				m_pEffect;				// D3DX effect interface
	ID3D10EffectTechnique*		m_RenderTechnique;
	ID3D10EffectTechnique*		m_DisplayTechnique;
	ID3D10EffectTechnique*		m_HButterflyTechnique;
	ID3D10EffectTechnique*		m_VButterflyTechnique;
	ID3D10EffectTechnique*		m_ScaleTechnique;

	ID3D10EffectScalarVariable*	m_BflyPassNumVar;

	ID3D10EffectTechnique*				m_CurrentTechnique;

	ID3D10EffectShaderResourceVariable* m_SrcImageResource;
	ID3D10EffectShaderResourceVariable* m_BflyValsResource;

	ID3D10Buffer*						m_VertexBuffer;
	ID3D10InputLayout*					m_VertexLayout;

	ID3D10Device*						m_D3DDevice;

	int	m_Width;
	int m_Height;
	int m_NumButterfliesX;
	int m_NumButterfliesY;
	//FFTType	m_FFTType;
	bool m_isPingRenderTarget;

	inline void SetPingAsRenderTarget()
	{
		m_isPingRenderTarget = false;
		m_PingRTTexture.SetTextureAsRenderTargetAndClear();
		m_SrcImageResource->SetResource(m_PongRTTexture.GetResourceView());
	}

	inline void SetPongAsRenderTarget()
	{
		m_isPingRenderTarget = true;
		m_PongRTTexture.SetTextureAsRenderTargetAndClear();
		m_SrcImageResource->SetResource(m_PingRTTexture.GetResourceView());
	}

	void PingPongRenderTargetAndSourceTexture();

public:
	FFTDX10();
	virtual ~FFTDX10()
	{
		SAFE_RELEASE(m_pEffect);
		SAFE_RELEASE(m_VertexLayout);
		SAFE_RELEASE(m_VertexBuffer);
	}

	HRESULT CreateVertexBuffer();

	int Run();
	//void SetFFT(FFTType fft) { m_FFTType = fft; }
	virtual void InitApp();
	// Messaging Events
	virtual LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, bool* pbNoFurtherProcessing );
	virtual void    CALLBACK KeyboardProc( UINT nChar, bool bKeyDown, bool bAltDown );
	virtual void    CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext  );

	// Device Creation & Deletion
	virtual bool	CALLBACK IsDeviceAcceptable( UINT Adapter, UINT Output, D3D10_DRIVER_TYPE DeviceType, DXGI_FORMAT BackBufferFormat, bool bWindowed );
	virtual HRESULT CALLBACK OnCreateDevice( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc );
	virtual bool	CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings );
	virtual bool	CALLBACK OnDeviceRemoved( );
	virtual void    CALLBACK OnDestroyDevice( );

	virtual void Render(ID3D10EffectTechnique* pTechnique);
	virtual void Render();

	// DX10 Specific
	virtual HRESULT CALLBACK OnSwapChainResized( ID3D10Device* pd3dDevice, IDXGISwapChain *pSwapChain, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc );
	virtual void    CALLBACK OnSwapChainReleasing(  );
	// Note no Lost device Event f
	// FrameUpdate
	virtual void    CALLBACK OnFrameMove  ( double fTime, float fElapsedTime );
	virtual void    CALLBACK OnFrameRender( ID3D10Device* pd3dDevice, double fTime, float fElapsedTime );
	void			Butterfly(ID3D10EffectTechnique* pTechnique, int numPasses);
};

